// SMF

#pragma once

#include <set>

#include "Constants.h"
#include "Interface.h"
#include "Times.h"
#include "SmfInterpreter.h"


namespace GuruGuruSmf { namespace Core { namespace Analyzer {
	
	/// <summary>
	/// SMFCxgsp̊NX(Analyzer)
	/// </summary>
	class SmfEventExecutor
	{
	public:
		// sI
		virtual void Execute(SmfEvent* smfEvent, IAnalyzerCore* core) = 0;
	};
	
	/// <summary>
	/// sȂp
	/// </summary>
	class NullEventExecutor : public SmfEventExecutor
	{
	public:
		void Execute(SmfEvent* smfEvent, IAnalyzerCore* core){}
	};
	
	/// <summary>
	/// m[gIp
	/// </summary>
	class NoteOnEventExecutor : public SmfEventExecutor
	{
	public:
		void Execute(SmfEvent* smfEvent, IAnalyzerCore* core);
	};
	
	/// <summary>
	/// m[gItp
	/// </summary>
	class NoteOffEventExecutor : public SmfEventExecutor
	{
	public:
		void Execute(SmfEvent* smfEvent, IAnalyzerCore* core);
	};
	
	/// <summary>
	/// e|p
	/// </summary>
	class TempoEventExecutor : public SmfEventExecutor
	{
	public:
		void Execute(SmfEvent* smfEvent, IAnalyzerCore* core);
	};
	
	/// <summary>
	/// vO`FWp
	/// </summary>
	class ProgramChangeEventExecutor : public SmfEventExecutor
	{
	public:
		void Execute(SmfEvent* smfEvent, IAnalyzerCore* core);
	};
	
	/// <summary>
	/// Rg[`FWp
	/// </summary>
	class ControlChangeEventExecutor : public SmfEventExecutor
	{
	public:
		void Execute(SmfEvent* smfEvent, IAnalyzerCore* core);
	};
	
	
	/// <summary>
	/// GNXN[Vup
	/// </summary>
	class ExclusiveEventExecutor : public SmfEventExecutor
	{
	public:
		void Execute(SmfEvent* smfEvent, IAnalyzerCore* core);
	};
	
	
	/// <summary>
	/// SMF
	/// </summary>
	class Analyzer : public SmfInterpreter, public IAnalyzerCore
	{
	private:
		SmfEventExecutor** eventExecutors;
		AnalyzerTimes* times;
		int eventCount;
		SmfAnalyzedData* smfData;
		
		uint* voiceBank;
		std::set<uint> voiceList;

	protected:
		// Ticki߂ĉtԂvZ
		void UpdateTick(int tick);

		// Cxg
		void EventExecute(SmfEvent* smfEvent);

		// 
		void Initialize(Smf* newSmf);

		// `֊߂
		void Rewind();

	public:
		Analyzer();		// RXgN^
		~Analyzer();	// fXgN^

		//
		// IAnalyzerCore
		//
		
		// e|ݒ
		void SetTempo(int tempo){ times->SetTempo(tempo); }

		// m[gIm[gIt̃Cxg
		void NoteEvent(bool isNoteOn);

		// vO`FWCxg
		void ProgramChangeEvent(int channel, int voice);

		// oNZNgCxg
		void BankSelectEvent(int channel, bool isMsb, int bank);

		// Yp[gCxg
		void UseRhythmPartEvent(int channel, bool isRhythmPart);

		// {[Cxg
		void VolumeEvent(int volume);

		// [v|Cg𔭌
		void MarkLoopPoint();

		//

		// ́I
		GGSERROR Analyze(Smf* newSmf);


	};
	
	
	
	
	
	
}}}




